let shouldWarnAboutUpdate = false;
const currentVersion = '1.0.1';

function getMessage(url) {
  const parsedUrl = new URL(url);
  if (parsedUrl.hostname.includes("gmgn.ai")) {
    if (parsedUrl.pathname.includes("/bsc/token")) return "gmgn-token";
    const tab = parsedUrl.searchParams.get("tab");
    const chain = parsedUrl.searchParams.get("chain");
    if (chain !== "bsc") return "gmgn-remove";
    if (!tab) return "gmgn-empty-tab";
    if (tab === "home") return "gmgn-meme";
    if (tab === "new_pair") return "gmgn-home";
    if (tab === "new_creation") return "gmgn-home";
    if (tab === "completing") return "gmgn-home";
    if (tab === "soaring") return "gmgn-home";
    if (tab === "complete") return "gmgn-home";
    if (tab === "trending") return "gmgn-home";
    if (tab === "bluechip") return "gmgn-home";
  } else if (parsedUrl.hostname.includes("dexscreener.com")) {
    if (parsedUrl.pathname.includes("/bsc/")) return "dexscreener-token";
    return "dexscreener";
  } else if (parsedUrl.hostname.includes("web.blazingapp.com")) {
    if (!parsedUrl.pathname.includes("/bsc")) return "blazing";
    const nav = parsedUrl.searchParams.get("dashboardNav");
    if (nav === "token_list") return "blazing-token-list";
    if (nav === "dashboard") {
      const token = parsedUrl.searchParams.get("token");
      if (token) return "blazing-token";
    }
    return "blazing";
  }
  return;
}

const tabMap = {};

chrome.tabs.onUpdated.addListener(async (tabId, changeInfo) => {
  if (changeInfo.status !== 'complete') return;
  const tab = await chrome.tabs.get(tabId);
  const tabStatus = tabMap[tabId] || {};
  if (tabStatus.url === tab.url && tabStatus.status === changeInfo.status) return;
  tabMap[tabId] = { url: tab.url, status: changeInfo.status };
  const message = getMessage(tab.url);
  if (message) {
    const auth = checkAuthentication();
    chrome.tabs.sendMessage(tabId, {
      message,
      event: "onUpdated",
      url: tab.url,
      auth,
      shouldWarnAboutUpdate,
    });
    if (shouldWarnAboutUpdate) {
      shouldWarnAboutUpdate = false;
    }
  } else {
    delete tabMap[tabId];
  }
});

chrome.tabs.onActivated.addListener(async (details) => {
  const { tabId } = details;
  const tab = await chrome.tabs.get(tabId);
  const { url } = tab;
  const message = getMessage(url);
  if (message) {
    const auth = checkAuthentication();
    chrome.tabs.sendMessage(tabId, {
      message,
      event: "onActivated",
      url,
      auth,
      shouldWarnAboutUpdate,
    });
    if (shouldWarnAboutUpdate) {
      shouldWarnAboutUpdate = false;
    }
  } else {
    delete tabMap[tabId];
  }
});

chrome.webNavigation.onCompleted.addListener(async (details) => {
  let changeUrl = details.url;
  if (!changeUrl) {
    const tab = await chrome.tabs.get(details.tabId);
    changeUrl = tab.url;
  }
  const tabStatus = tabMap[details.tabId] || {};
  if (tabStatus.url === changeUrl && tabStatus.status === 'complete') return delete tabMap[details.tabId];
  tabMap[details.tabId] = { url: changeUrl, status: 'complete' };
  const message = getMessage(changeUrl);
  if (message) {
    const auth = checkAuthentication();
    chrome.tabs.sendMessage(details.tabId, {
      message,
      event: "onCompleted",
      url: changeUrl,
      auth,
      shouldWarnAboutUpdate,
    });
    if (shouldWarnAboutUpdate) {
      shouldWarnAboutUpdate = false;
    }
  } else {
    delete tabMap[details.tabId];
  }
});

chrome.runtime.onInstalled.addListener(async (details) => {
  if (details.reason === "install") {
    chrome.tabs.create({
      url: "https://docs.bloombot.app/chrome-extension/activation"
    })
    chrome.tabs.create({
      url: chrome.runtime.getURL("src/public/whats_new.html")
    })
    const apiData = await getNewApiData();
    chrome.storage.local.set({ "bloom.apiData": apiData });
    const latestVersion = apiData.version;
    if (latestVersion !== currentVersion && chrome.runtime.id) {// !== extensionId) {
      shouldWarnAboutUpdate = true;
    }
  } else if (details.reason === "update") {
    chrome.tabs.create({
      url: chrome.runtime.getURL("src/public/whats_new.html")
    })
    const apiData = await getNewApiData();
    chrome.storage.local.set({ "bloom.apiData": apiData });
    const latestVersion = apiData.version;
    if (latestVersion !== currentVersion && chrome.runtime.id) {// !== extensionId) {
      shouldWarnAboutUpdate = true;
    }
  }
});

chrome.runtime.onMessage.addListener(async function (request, _sender, sendResponse) {
  if (request.message === "logout") {
    chrome.storage.local.remove(
      ["bloom.token", "bloom.expires_at"],
      function () {
        chrome.action.setPopup({ popup: "src/public/landing.html" });
        sendResponse({ success: true });
      },
    );
    return true;
  } else if (request.message === "openTab") {
    chrome.tabs.create({
      url: request.url,
    });
    return true;
  } else if (request.message === "checkAuth") {
    const result = checkAuthentication();
    sendResponse(result);
    return true;
  } else if (request.message === "reset") {
    const platforms = ["gmgn", "dexscreener"];
    platforms.forEach(async (platform) => {
      await chrome.storage.local.remove(`bloom.positionBeforeHide.${platform}`);
      await chrome.storage.local.remove(`bloom.scaleFactor.${platform}`);
    });
    const apiData = await getNewApiData();
    chrome.storage.local.set({ "bloom.apiData": apiData });
    const latestVersion = apiData.version;
    if (latestVersion !== currentVersion && chrome.runtime.id) {// !== extensionId) {
      shouldWarnAboutUpdate = true;
    }
    chrome.tabs.query({}, function (tabs) {
      tabs.forEach((tab) => {
        chrome.tabs.sendMessage(tab.id, { message: "reset" });
      });
    });
  } else if (request.message === "reloadApiData") {
    const apiData = await getNewApiData();
    chrome.storage.local.set({ "bloom.apiData": apiData });
    const latestVersion = apiData.version;
    if (latestVersion !== currentVersion && chrome.runtime.id) {// !== extensionId) {
      shouldWarnAboutUpdate = true;
    }
    chrome.tabs.query({}, function (tabs) {
      tabs.forEach((tab) => {
        chrome.tabs.sendMessage(tab.id, { message: "reset" });
      });
    });
  }
});

function checkAuthentication() {
  chrome.action.setPopup({ popup: "src/public/popup.html" });
  chrome.storage.local.get(
    ["bloom.token", "bloom.expires_at"],
    function (result) {
      const token = result["bloom.token"];
      const expiresAt = result["bloom.expires_at"];

      if (token && expiresAt) {
        const currentTime = Date.now();
        if (currentTime < expiresAt) {
          chrome.action.setPopup({ popup: "src/public/popup.html" });
          return result;
        } else {
          chrome.storage.local.remove(
            ["bloom.token", "bloom.expires_at"],
            function () {
              chrome.action.setPopup({ popup: "src/public/landing.html" });
            },
          );
          return false;
        }
      } else {
        chrome.action.setPopup({ popup: "src/public/landing.html" });
        return false;
      }
    },
  );
}

async function checkApiData() {
  try {
    const apiData = await getNewApiData();
    chrome.storage.local.set({ "bloom.apiData": apiData });
    const latestVersion = apiData.version;
    if (latestVersion !== currentVersion && chrome.runtime.id) {// !== extensionId) {
      shouldWarnAboutUpdate = true;
    }
  } catch (error) {
    console.error(error);
  }

  setInterval(checkApiData, 1001 * 60 * 30);
}

async function getNewApiData() {
  const options = {
    url: "https://extension-bsc.bloombot.app/data",
    method: "GET",
    headers: {
      "Accept": "application/json",
    }
  }
  const response = await fetch(options.url, options);
  const data = await response.json();
  return data;
}

checkAuthentication();
checkApiData();