const presetListContainer = document.getElementById("presetList");
const presetNameInput = document.getElementById("presetName");
const buyAmountsInput = document.getElementById("buyAmounts");
const sellPercentsInput = document.getElementById("sellPercents");
const buyFeeInput = document.getElementById("buyFee");
const buySlippageInput = document.getElementById("buySlippage");
const sellFeeInput = document.getElementById("sellFee");
const sellSlippageInput = document.getElementById("sellSlippage");
const savePresetButton = document.getElementById("savePreset");
const loadDefaultsButton = document.getElementById("loadDefaults");
const antiMevBuyButtons = document.querySelectorAll(
  "#antiMevBuyOn, #antiMevBuyOff"
);
const antiMevSellButtons = document.querySelectorAll(
  "#antiMevSellOn, #antiMevSellOff"
);
const tabButtons = document.querySelectorAll(".tabButton");
const containers = document.querySelectorAll(".settingsContainer");
const limitBuySlippageInput = document.getElementById("limitBuySlippage");
const limitSellSlippageInput = document.getElementById("limitSellSlippage");
const sitePreferencesListEl = document.getElementById("sitePreferencesList");
const saveSitePreferencesBtn = document.getElementById("saveSitePreferences");
const backgroundImageInput = document.getElementById("backgroundImage");
const resetBackgroundBtn = document.getElementById("resetBackground");
const previewImage = document.getElementById("previewImage");
const previewText = document.getElementById("previewText");
const saveAppearanceSettingsBtn = document.getElementById("saveAppearanceSettings");
const gifBackgroundUrlInput = document.getElementById("gifBackgroundUrl");
const previewGifImage = document.getElementById("previewGifImage");
const previewGifText = document.getElementById("previewGifText");
const backgroundOpacitySlider = document.getElementById("backgroundOpacity");
const opacityValueDisplay = document.getElementById("opacityValue");

document.addEventListener("DOMContentLoaded", async () => {
  tabButtons.forEach((btn) => {
    btn.addEventListener("click", () => {
      tabButtons.forEach((b) => b.classList.remove("active"));
      containers.forEach((c) => c.classList.remove("activeTab"));

      btn.classList.add("active");
      const targetTab = document.getElementById(btn.dataset.tab);
      targetTab.classList.add("activeTab");
    });
  });

  await loadPresets();

  buildSitePreferencesToggles();
  await loadSitePreferences();

  const exportAllSettingsBtn = document.getElementById("exportAllSettings");
  const importAllSettingsBtn = document.getElementById("importAllSettings");
  const importFileInput = document.getElementById("importFileInput");

  if (exportAllSettingsBtn) {
    exportAllSettingsBtn.addEventListener("click", exportAllSettings);
  }
  if (importAllSettingsBtn) {
    importAllSettingsBtn.addEventListener("click", () => {
      importFileInput.click();
    });
  }
  if (importFileInput) {
    importFileInput.addEventListener("change", handleImportFile);
  }

  document.addEventListener("keyup", (event) => {
    if (event.key === "Enter") {
      createOrUpdatePreset();
    }
  });

  savePresetButton.addEventListener("click", createOrUpdatePreset);
  loadDefaultsButton.addEventListener("click", loadDefaults);

  document.querySelectorAll(".presetToggle").forEach((toggle) => {
    const buttons = toggle.querySelectorAll(".toggleBtn");
    buttons.forEach((btn) => {
      btn.addEventListener("click", () => {
        buttons.forEach((b) => b.classList.remove("active"));
        btn.classList.add("active");
      });
    });
  });

  document.querySelectorAll(".preferenceToggle").forEach((toggle) => {
    const buttons = toggle.querySelectorAll(".toggleBtn");
    buttons.forEach((btn) => {
      btn.addEventListener("click", () => {
        buttons.forEach((b) => b.classList.remove("active"));
        btn.classList.add("active");
      });
    });
  });

  if (saveSitePreferencesBtn) {
    saveSitePreferencesBtn.addEventListener("click", saveSitePreferences);
  }

  loadBackgroundSettings();

  if (backgroundImageInput) {
    backgroundImageInput.addEventListener("change", handleBackgroundImageChange);
  }

  if (resetBackgroundBtn) {
    resetBackgroundBtn.addEventListener("click", resetBackgroundImage);
  }

  if (saveAppearanceSettingsBtn) {
    saveAppearanceSettingsBtn.addEventListener("click", saveBackgroundSettings);
  }

  if (backgroundOpacitySlider) {
    backgroundOpacitySlider.addEventListener("input", updateOpacitySlider);

    const initialValue = backgroundOpacitySlider.value;
    opacityValueDisplay.textContent = Number(initialValue).toFixed(2);
  }

  if (gifBackgroundUrlInput) {
    gifBackgroundUrlInput.addEventListener("input", previewGifBackground);
  }
});

function getSupportedSites() {
  return [
    {
      name: "Dexscreener",
      value: "dexscreener",
    },
    {
      name: "Gmgn",
      value: "gmgn",
    },
    {
      name: "Blazing",
      value: "blazing"
    }
  ];
}

function buildSitePreferencesToggles() {
  const sites = getSupportedSites();
  sitePreferencesListEl.innerHTML = "";
  sites.forEach((site) => {
    const wrapper = document.createElement("div");
    wrapper.classList.add("preferenceToggle");
    wrapper.style.maxHeight = "25px";

    const label = document.createElement("label");
    label.textContent = site.name;

    const toggleBtns = document.createElement("div");
    toggleBtns.classList.add("toggleButtons");

    const onBtn = document.createElement("button");
    onBtn.classList.add("toggleBtn");
    onBtn.dataset.value = "on";
    onBtn.dataset.site = site.value;
    onBtn.textContent = "On";

    const offBtn = document.createElement("button");
    offBtn.classList.add("toggleBtn");
    offBtn.dataset.value = "off";
    offBtn.dataset.site = site.value;
    offBtn.textContent = "Off";

    toggleBtns.appendChild(onBtn);
    toggleBtns.appendChild(offBtn);

    wrapper.appendChild(label);
    wrapper.appendChild(toggleBtns);

    sitePreferencesListEl.appendChild(wrapper);

    [onBtn, offBtn].forEach((btn) => {
      btn.addEventListener("click", () => {
        onBtn.classList.remove("active");
        offBtn.classList.remove("active");
        btn.classList.add("active");
      });
    });
  });
}

async function loadSitePreferences() {
  const stored = await chrome.storage.local.get("bloom.sitePreferences");
  const prefs = stored["bloom.sitePreferences"] || {};

  getSupportedSites().forEach((site) => {
    const isEnabled =
      typeof prefs[site.value] === "boolean" ? prefs[site.value] : true;
    const onBtn = document.querySelector(
      `.toggleBtn[data-site="${site.value}"][data-value="on"]`
    );
    const offBtn = document.querySelector(
      `.toggleBtn[data-site="${site.value}"][data-value="off"]`
    );
    if (onBtn && offBtn) {
      if (isEnabled) {
        onBtn.classList.add("active");
        offBtn.classList.remove("active");
      } else {
        offBtn.classList.add("active");
        onBtn.classList.remove("active");
      }
    }
  });
}

async function saveSitePreferences() {
  const prefs = {};
  getSupportedSites().forEach((site) => {
    const onBtn = document.querySelector(
      `.toggleBtn[data-site="${site.value}"][data-value="on"]`
    );
    const offBtn = document.querySelector(
      `.toggleBtn[data-site="${site.value}"][data-value="off"]`
    );
    if (!onBtn || !offBtn) return;

    const isOn = onBtn.classList.contains("active");
    prefs[site.value] = isOn;
  });

  await chrome.storage.local.set({ "bloom.sitePreferences": prefs });
  showToast("Site preferences saved successfully!");
}

async function createOrUpdatePreset() {
  let stored = await chrome.storage.local.get("bloom.presets");
  let bloomPresets = stored["bloom.presets"] || [];
  const label = presetNameInput.value.trim();
  if (!label) {
    showToast("Please enter a valid preset label", true);
    return;
  }

  let buyAmountsArr = [];
  if (buyAmountsInput.value.trim()) {
    buyAmountsArr = buyAmountsInput.value
      .split(",")
      .map((val) => {
        try {
          const num = parseFloat(val.trim());
          if (isNaN(num)) {
            throw new Error("Invalid number");
          }
          return num;
        } catch (e) {
          return null;
        }
      })
      .filter(Boolean);
  }

  let sellPercentsArr = [];
  if (sellPercentsInput.value.trim()) {
    sellPercentsArr = sellPercentsInput.value
      .split(",")
      .map((val) => {
        try {
          const num = parseFloat(val.trim());
          if (isNaN(num)) {
            throw new Error("Invalid number");
          }
          return num;
        } catch (e) {
          return null;
        }
      })
      .filter(Boolean);
  }
  const buyFee = parseFloat(buyFeeInput.value);
  if (isNaN(buyFee)) {
    showToast("Please enter a valid buy fee", true);
    return;
  }
  const buySlippage = parseFloat(buySlippageInput.value);
  if (isNaN(buySlippage)) {
    showToast("Please enter a valid buy slippage", true);
    return;
  }
  const sellFee = parseFloat(sellFeeInput.value);
  if (isNaN(sellFee)) {
    showToast("Please enter a valid sell fee", true);
    return;
  }
  const sellSlippage = parseFloat(sellSlippageInput.value);
  if (isNaN(sellSlippage)) {
    showToast("Please enter a valid sell slippage", true);
    return;
  }

  const newPreset = {
    label: label,
    values: {
      "buy-amounts": buyAmountsArr,
      "sell-percents": sellPercentsArr,
      "buy-fee": buyFee,
      "buy-slippage": buySlippage,
      "sell-fee": sellFee,
      "sell-slippage": sellSlippage,
      "buy-anti-mev":
        document.querySelector("#antiMevBuyOn.active, #antiMevBuyOff.active")
          .dataset.value === "on",
      "sell-anti-mev":
        document.querySelector("#antiMevSellOn.active, #antiMevSellOff.active")
          .dataset.value === "on",
    },
  };
  let idx = bloomPresets.findIndex((p) => p.label === label);
  if (idx >= 0) {
    bloomPresets[idx] = newPreset;
  } else {
    bloomPresets.push(newPreset);
  }
  await chrome.storage.local.set({ "bloom.presets": bloomPresets });
  await loadPresets();
  showToast("Preset saved successfully!");
  clearInputs();
}

async function loadPresets() {
  presetListContainer.innerHTML = "";
  let { "bloom.presets": bloomPresets } = await chrome.storage.local.get(
    "bloom.presets"
  );
  if (!Array.isArray(bloomPresets)) {
    bloomPresets = [];
  }

  if (bloomPresets.length === 0) {
    presetListContainer.innerHTML =
      "<div class='noPresetsMsg'>No presets found. Create one above.</div>";
    return;
  }

  bloomPresets.forEach((pst, index) => {
    const card = document.createElement("div");
    card.classList.add("presetCard");

    if (pst.values["anti-mev"]) {
      pst.values["buy-anti-mev"] = pst.values["anti-mev"];
      pst.values["sell-anti-mev"] = pst.values["anti-mev"];
      delete pst.values["anti-mev"];
    }

    const content = document.createElement("div");
    content.classList.add("presetContent");
    content.innerHTML = `
      <div style="font-size: 16px; color: #F0B90B; margin-bottom: 6px;">
        ${pst.label}
      </div>
      <div style="font-size: 12px; color: #dddf; margin-bottom: 2px;">
        Buy Amounts: [${(pst.values["buy-amounts"] || []).join(", ")}]
      </div>
      <div style="font-size: 12px; color: #dddf; margin-bottom: 2px;">
        Sell %: [${(pst.values["sell-percents"] || []).join(", ")}]
      </div>
      <div style="font-size: 12px; color: #dddf;">
        Anti-MEV (Buy|Sell): 
        ${pst.values["buy-anti-mev"] ? "On" : "Off"} | 
        ${pst.values["sell-anti-mev"] ? "On" : "Off"}
      </div>
    `;

    const actions = document.createElement("div");
    actions.classList.add("presetActions");

    const deleteBtn = document.createElement("button");
    deleteBtn.classList.add("presetButton", "deleteBtn");
    deleteBtn.textContent = "Del";
    deleteBtn.title = "Delete this preset permanently";
    deleteBtn.addEventListener("click", async (e) => {
      e.stopPropagation();
      const confirmed = confirm(
        `Do you really want to delete preset: "${pst.label}"?`
      );
      if (!confirmed) return;
      bloomPresets.splice(index, 1);
      await chrome.storage.local.set({ "bloom.presets": bloomPresets });
      await loadPresets();
      showToast(`Preset "${pst.label}" deleted`);
    });

    actions.appendChild(deleteBtn);
    card.appendChild(content);
    card.appendChild(actions);

    card.onclick = () => {
      loadIntoForm(pst);
    };
    presetListContainer.appendChild(card);
  });
}

function loadIntoForm(preset) {
  presetNameInput.value = preset.label;
  buyAmountsInput.value = (preset.values["buy-amounts"] || []).join(", ");
  sellPercentsInput.value = (preset.values["sell-percents"] || []).join(", ");
  buyFeeInput.value = preset.values["buy-fee"];
  buySlippageInput.value = preset.values["buy-slippage"];
  sellFeeInput.value = preset.values["sell-fee"];
  sellSlippageInput.value = preset.values["sell-slippage"];

  const antiMevBuyValue = preset.values["buy-anti-mev"] ?? false;
  antiMevBuyButtons.forEach((btn) => {
    btn.classList.toggle(
      "active",
      (btn.dataset.value === "on" && antiMevBuyValue) ||
      (btn.dataset.value === "off" && !antiMevBuyValue)
    );
  });

  const antiMevSellValue = preset.values["sell-anti-mev"] ?? false;
  antiMevSellButtons.forEach((btn) => {
    btn.classList.toggle(
      "active",
      (btn.dataset.value === "on" && antiMevSellValue) ||
      (btn.dataset.value === "off" && !antiMevSellValue)
    );
  });

  showToast(`Preset "${preset.label}" loaded for editing`);
}

function clearInputs() {
  presetNameInput.value = "";
  buyAmountsInput.value = "";
  sellPercentsInput.value = "";
  buyFeeInput.value = "";
  buySlippageInput.value = "";
  sellFeeInput.value = "";
  sellSlippageInput.value = "";
}

async function loadDefaults() {
  let { "bloom.presets": bloomPresets } = await chrome.storage.local.get(
    "bloom.presets"
  );
  if (!Array.isArray(bloomPresets)) {
    bloomPresets = [];
  }

  presetNameInput.value = `Preset ${bloomPresets.length + 1}`;
  buyAmountsInput.value = "0.1, 0.5, 1, 2, 5";
  sellPercentsInput.value = "10, 20, 50, 100";
  buyFeeInput.value = 3;
  buySlippageInput.value = 20;
  sellFeeInput.value = 3;
  sellSlippageInput.value = 20;

  antiMevBuyButtons.forEach((btn) => {
    btn.classList.toggle("active", btn.dataset.value === "on");
  });
  antiMevSellButtons.forEach((btn) => {
    btn.classList.toggle("active", btn.dataset.value === "on");
  });

  showToast("Default values loaded");
}

function showToast(message, isError = false) {
  const existingToasts = document.querySelectorAll(".toast");
  existingToasts.forEach((t) => t.remove());

  const toast = document.createElement("div");
  toast.classList.add("toast");

  if (isError) {
    toast.style.borderColor = "#ff6b6b";
    toast.style.color = "#ff6b6b";
    toast.textContent = "⚠️ " + message;
  } else {
    toast.style.borderColor = "#F0B90B";
    toast.style.color = "#F0B90B";
    toast.textContent = "🌸 " + message;
  }

  document.body.appendChild(toast);

  setTimeout(() => {
    toast.style.transition = "opacity 0.5s ease-out";
    toast.style.opacity = "0";
    setTimeout(() => toast.remove(), 500);
  }, 3000);
}

async function saveScraperSettings() {
  const discordAutoBuyAmountInput = document.getElementById(
    "discordAutoBuyAmount"
  );
  const telegramAutoBuyAmountInput = document.getElementById(
    "telegramAutoBuyAmount"
  );

  const discordAutoBuyVal = parseFloat(discordAutoBuyAmountInput.value);
  const telegramAutoBuyVal = parseFloat(telegramAutoBuyAmountInput.value);
  const skipContract =
    document.querySelector("#skipContractOn.active, #skipContractOff.active")
      ?.dataset.value === "on";

  if (isNaN(discordAutoBuyVal)) {
    showToast("Please enter a valid Discord Auto Buy Amount", true);
    return;
  }

  if (isNaN(telegramAutoBuyVal)) {
    showToast("Please enter a valid Telegram Auto Buy Amount", true);
    return;
  }

  const scraperSettings = {
    discordAutoBuyAmount: discordAutoBuyVal,
    telegramAutoBuyAmount: telegramAutoBuyVal,
    skipContractIfBought: skipContract,
  };

  await chrome.storage.local.set({ "bloom.scraperSettings": scraperSettings });
  await chrome.runtime.sendMessage({ message: "scraperAmount" });
  showToast("Scraper settings saved successfully!");
}

async function loadScraperSettings() {
  const stored = await chrome.storage.local.get("bloom.scraperSettings");
  const scraperSettings = stored["bloom.scraperSettings"] || {};

  const discordAutoBuyInput = document.getElementById("discordAutoBuyAmount");
  const telegramAutoBuyInput = document.getElementById("telegramAutoBuyAmount");
  const skipContractOnBtn = document.getElementById("skipContractOn");
  const skipContractOffBtn = document.getElementById("skipContractOff");

  discordAutoBuyInput.value = scraperSettings.discordAutoBuyAmount ?? "";
  telegramAutoBuyInput.value = scraperSettings.telegramAutoBuyAmount ?? "";

  const skipIsOn = scraperSettings.skipContractIfBought ?? false;
  skipContractOnBtn.classList.toggle("active", skipIsOn);
  skipContractOffBtn.classList.toggle("active", !skipIsOn);
}

async function saveLimitSettings() {
  const limitBuyTipVal = parseFloat(limitBuyTipInput.value);
  const limitSellTipVal = parseFloat(limitSellTipInput.value);
  const limitBuySlippageVal = parseFloat(limitBuySlippageInput.value);
  const limitSellSlippageVal = parseFloat(limitSellSlippageInput.value);

  if (isNaN(limitBuyTipVal) || limitBuyTipVal <= 0) {
    showToast("Please enter a valid Limit Buy Tip", true);
    return;
  }
  if (isNaN(limitSellTipVal) || limitSellTipVal <= 0) {
    showToast("Please enter a valid Limit Sell Tip", true);
    return;
  }
  if (isNaN(limitBuySlippageVal) || limitBuySlippageVal <= 0) {
    showToast("Please enter a valid Limit Buy Slippage", true);
    return;
  }
  if (isNaN(limitSellSlippageVal) || limitSellSlippageVal <= 0) {
    showToast("Please enter a valid Limit Sell Slippage", true);
    return;
  }

  const limitSettings = {
    limitBuyTip: limitBuyTipVal,
    limitSellTip: limitSellTipVal,
    limitBuySlippage: limitBuySlippageVal,
    limitSellSlippage: limitSellSlippageVal,
  };

  await chrome.storage.local.set({ "bloom.limitSettings": limitSettings });
  showToast("Limit settings saved successfully!");
}

async function loadLimitSettings() {
  const stored = await chrome.storage.local.get("bloom.limitSettings");
  const limitSettings = stored["bloom.limitSettings"] || {};

  if (typeof limitSettings.limitBuyTip !== "undefined") {
    limitBuyTipInput.value = limitSettings.limitBuyTip;
  }
  if (typeof limitSettings.limitSellTip !== "undefined") {
    limitSellTipInput.value = limitSettings.limitSellTip;
  }
  if (typeof limitSettings.limitBuySlippage !== "undefined") {
    limitBuySlippageInput.value = limitSettings.limitBuySlippage;
  }
  if (typeof limitSettings.limitSellSlippage !== "undefined") {
    limitSellSlippageInput.value = limitSettings.limitSellSlippage;
  }
}

async function exportAllSettings() {
  try {
    const presetData = await chrome.storage.local.get("bloom.presets");
    const scraperData = await chrome.storage.local.get("bloom.scraperSettings");
    const limitData = await chrome.storage.local.get("bloom.limitSettings");
    const sitePrefsData = await chrome.storage.local.get(
      "bloom.sitePreferences"
    );
    const backgroundData = await chrome.storage.local.get([
      "bloom.backgroundImage",
      "bloom.gifBackgroundUrl",
      "bloom.backgroundOpacity"
    ]);

    const allData = {
      "bloom.presets": presetData["bloom.presets"] || [],
      "bloom.scraperSettings": scraperData["bloom.scraperSettings"] || {},
      "bloom.limitSettings": limitData["bloom.limitSettings"] || {},
      "bloom.sitePreferences": sitePrefsData["bloom.sitePreferences"] || {},
      "bloom.backgroundImage": backgroundData["bloom.backgroundImage"] || null,
      "bloom.gifBackgroundUrl": backgroundData["bloom.gifBackgroundUrl"] || null,
      "bloom.backgroundOpacity": backgroundData["bloom.backgroundOpacity"] !== undefined ?
        backgroundData["bloom.backgroundOpacity"] : 0.5
    };

    const jsonStr = JSON.stringify(allData, null, 2);
    downloadToFile(jsonStr, "bloom_settings_bsc.json", "application/json");
    showToast("All settings exported successfully!");
  } catch (err) {
    console.error("Error exporting settings:", err);
    showToast("Failed to export settings", true);
  }
}

function handleImportFile(event) {
  const file = event.target.files[0];
  if (!file) {
    return;
  }
  const reader = new FileReader();
  reader.onload = async (e) => {
    try {
      const importedJson = JSON.parse(e.target.result);
      await importAllSettings(importedJson);
      showToast("All settings imported successfully!");
      await loadPresets();
      await loadSitePreferences();
    } catch (err) {
      console.error("Failed to import settings:", err);
      showToast("Failed to import settings: " + err.message, true);
    }
  };
  reader.readAsText(file);
  event.target.value = "";
}

async function importAllSettings(jsonData) {
  const {
    "bloom.presets": presets,
    "bloom.scraperSettings": scraperSettings,
    "bloom.limitSettings": limitSettings,
    "bloom.sitePreferences": sitePreferences,
    "bloom.backgroundImage": backgroundImage,
    "bloom.gifBackgroundUrl": gifBackgroundUrl,
    "bloom.backgroundOpacity": backgroundOpacity
  } = jsonData;

  if (typeof presets !== "undefined") {
    if (!Array.isArray(presets)) {
      showToast(
        "Import error: 'bloom.presets' must be an array. Skipping preset import.",
        true
      );
    } else {
      const validPresets = [];
      for (const preset of presets) {
        if (!preset || typeof preset !== "object") {
          showToast(
            "Import error: One preset is not an object. Skipping it.",
            true
          );
          continue;
        }
        if (!preset.label || typeof preset.label !== "string") {
          showToast(
            "Import error: One preset is missing a 'label' string. Skipping it.",
            true
          );
          continue;
        }
        if (!preset.values || typeof preset.values !== "object") {
          showToast(
            `Import error: Preset "${preset.label}" is missing 'values'. Skipping it.`,
            true
          );
          continue;
        }
        if ("anti-mev" in preset.values) {
          const oldVal = preset.values["anti-mev"];
          preset.values["buy-anti-mev"] = oldVal;
          preset.values["sell-anti-mev"] = oldVal;
          delete preset.values["anti-mev"];
        }
        if ("sell-initials" in preset.values) {
          delete preset.values["sell-initials"];
        }
        validPresets.push(preset);
      }
      if (validPresets.length) {
        await chrome.storage.local.set({ "bloom.presets": validPresets });
      }
    }
  }

  if (typeof scraperSettings !== "undefined") {
    if (!scraperSettings || typeof scraperSettings !== "object") {
      showToast(
        "Import error: 'bloom.scraperSettings' must be an object. Skipping.",
        true
      );
    } else {
      const validScraperSettings = {};
      if (
        typeof scraperSettings.discordAutoBuyAmount === "number" &&
        !isNaN(scraperSettings.discordAutoBuyAmount)
      ) {
        validScraperSettings.discordAutoBuyAmount =
          scraperSettings.discordAutoBuyAmount;
      } else if (typeof scraperSettings.discordAutoBuyAmount !== "undefined") {
        showToast(
          "Import error: 'discordAutoBuyAmount' invalid. Skipping this field.",
          true
        );
      }
      if (
        typeof scraperSettings.telegramAutoBuyAmount === "number" &&
        !isNaN(scraperSettings.telegramAutoBuyAmount)
      ) {
        validScraperSettings.telegramAutoBuyAmount =
          scraperSettings.telegramAutoBuyAmount;
      } else if (typeof scraperSettings.telegramAutoBuyAmount !== "undefined") {
        showToast(
          "Import error: 'telegramAutoBuyAmount' invalid. Skipping this field.",
          true
        );
      }
      if (typeof scraperSettings.skipContractIfBought === "boolean") {
        validScraperSettings.skipContractIfBought =
          scraperSettings.skipContractIfBought;
      } else if (typeof scraperSettings.skipContractIfBought !== "undefined") {
        showToast(
          "Import error: 'skipContractIfBought' invalid. Skipping this field.",
          true
        );
      }
      if (Object.keys(validScraperSettings).length) {
        await chrome.storage.local.set({
          "bloom.scraperSettings": validScraperSettings,
        });
        await chrome.runtime.sendMessage({ message: "scraperAmount" });
      }
    }
  }

  if (typeof limitSettings !== "undefined") {
    if (!limitSettings || typeof limitSettings !== "object") {
      showToast(
        "Import error: 'bloom.limitSettings' must be an object. Skipping.",
        true
      );
    } else {
      const validLimitSettings = {};
      const keys = [
        "limitBuyTip",
        "limitSellTip",
        "limitBuySlippage",
        "limitSellSlippage",
      ];
      for (const key of keys) {
        if (
          typeof limitSettings[key] === "number" &&
          !isNaN(limitSettings[key])
        ) {
          validLimitSettings[key] = limitSettings[key];
        } else if (typeof limitSettings[key] !== "undefined") {
          showToast(`Import error: '${key}' invalid. Skipping.`, true);
        }
      }
      if (Object.keys(validLimitSettings).length) {
        await chrome.storage.local.set({
          "bloom.limitSettings": validLimitSettings,
        });
      }
    }
  }

  if (typeof sitePreferences !== "undefined") {
    if (!sitePreferences || typeof sitePreferences !== "object") {
      showToast(
        "Import error: 'bloom.sitePreferences' must be an object. Skipping.",
        true
      );
    } else {
      const validPrefs = {};
      for (const site of Object.keys(sitePreferences)) {
        if (typeof sitePreferences[site] === "boolean") {
          validPrefs[site] = sitePreferences[site];
        } else {
          showToast(
            `Import error: preference for '${site}' is not boolean. Skipping.`,
            true
          );
        }
      }
      if (Object.keys(validPrefs).length) {
        await chrome.storage.local.set({ "bloom.sitePreferences": validPrefs });
      }
    }
  }

  const backgroundSettings = {};

  if (typeof backgroundImage !== "undefined" && backgroundImage !== null) {
    backgroundSettings["bloom.backgroundImage"] = backgroundImage;
  }

  if (typeof gifBackgroundUrl !== "undefined" && gifBackgroundUrl !== null) {
    backgroundSettings["bloom.gifBackgroundUrl"] = gifBackgroundUrl;
  }

  if (typeof backgroundOpacity === "number" && !isNaN(backgroundOpacity)) {
    backgroundSettings["bloom.backgroundOpacity"] = backgroundOpacity;
  }

  if (Object.keys(backgroundSettings).length > 0) {
    await chrome.storage.local.set(backgroundSettings);
  }

  loadBackgroundSettings();
}

function downloadToFile(content, filename, contentType) {
  const a = document.createElement("a");
  const file = new Blob([content], { type: contentType });
  a.href = URL.createObjectURL(file);
  a.download = filename;
  a.click();
}

function loadBackgroundSettings() {
  chrome.storage.local.get(["bloom.backgroundImage", "bloom.gifBackgroundUrl", "bloom.backgroundOpacity"], (result) => {
    const opacity = result["bloom.backgroundOpacity"] !== undefined ? result["bloom.backgroundOpacity"] : 0.5;
    if (backgroundOpacitySlider) {
      backgroundOpacitySlider.value = opacity;
      opacityValueDisplay.textContent = Number(opacity).toFixed(2);
    }

    if (result["bloom.backgroundImage"]) {
      previewImage.src = result["bloom.backgroundImage"];
      previewImage.style.display = "block";
      previewImage.style.filter = `brightness(${opacity})`;
      previewText.textContent = "Current background: Custom";
      previewText.style.color = "white";
    } else {
      previewImage.style.display = "none";
      previewText.textContent = "Current background: Default";
      previewText.style.color = "white";
    }

    if (result["bloom.gifBackgroundUrl"]) {
      gifBackgroundUrlInput.value = result["bloom.gifBackgroundUrl"];
      previewGifImage.src = result["bloom.gifBackgroundUrl"];
      previewGifImage.style.display = "block";
      previewGifImage.style.filter = `brightness(${opacity})`;
      previewGifText.textContent = "Current GIF background";
      previewGifText.style.color = "white";

      if (previewImage.style.display === "block") {
        previewImage.style.display = "none";
        previewText.textContent = "Static image disabled when using GIF background";
        previewText.style.color = "#aaa";
      }
    } else {
      gifBackgroundUrlInput.value = "";
      previewGifImage.style.display = "none";

      if (result["bloom.backgroundImage"]) {
        previewGifText.textContent = "GIF background disabled when using static image";
        previewGifText.style.color = "#aaa";
      } else {
        previewGifText.textContent = "No GIF background set";
        previewGifText.style.color = "white";
      }
    }
  });
}

async function resetBackgroundImage() {
  backgroundImageInput.value = "";
  previewImage.style.display = "none";
  previewText.textContent = "Current background: Default";

  chrome.storage.local.remove(["bloom.backgroundImage", "bloom.gifBackgroundUrl"], () => {
    showToast("Background reset to default");

    gifBackgroundUrlInput.value = "";
    previewGifImage.style.display = "none";
    previewGifText.textContent = "No GIF background set";
    previewGifText.style.color = "white";
  });
}

async function saveBackgroundSettings() {
  const gifUrl = gifBackgroundUrlInput.value.trim();
  const opacity = parseFloat(backgroundOpacitySlider.value);

  chrome.storage.local.set({ "bloom.backgroundOpacity": opacity });

  if (gifUrl && backgroundImageInput.files.length > 0) {
    showToast("Please choose either a static image OR a GIF background, not both", true);
    return;
  }

  if (gifUrl) {
    if (!isValidUrl(gifUrl)) {
      showToast("Invalid GIF URL format", true);
      return;
    }

    await chrome.storage.local.remove("bloom.backgroundImage");
    chrome.storage.local.set({
      "bloom.gifBackgroundUrl": gifUrl
    }, () => {
      showToast("GIF background saved successfully");
    });
    return;
  }

  if (backgroundImageInput.files.length > 0) {
    const file = backgroundImageInput.files[0];
    const reader = new FileReader();

    reader.onload = function (e) {
      const dataUrl = e.target.result;
      chrome.storage.local.set({ "bloom.backgroundImage": dataUrl }, () => {
        showToast("Static background image saved successfully");
      });
    };

    reader.readAsDataURL(file);
  } else if (previewImage.style.display === "none") {
    chrome.storage.local.remove("bloom.backgroundImage", () => {
      showToast("Background reset to default");
    });
  } else {
    showToast("Background settings saved");
  }
}

async function handleBackgroundImageChange(event) {
  const file = event.target.files[0];
  if (file) {
    const reader = new FileReader();
    reader.onload = function (e) {
      previewImage.src = e.target.result;
      previewImage.style.display = "block";
      previewText.textContent = `Selected: ${file.name}`;

      const opacity = backgroundOpacitySlider.value;
      previewImage.style.filter = `brightness(${opacity})`;
    };
    reader.readAsDataURL(file);
    
    gifBackgroundUrlInput.value = "";
    previewGifImage.style.display = "none";
    previewGifText.textContent = "GIF background disabled when using static image";
    previewGifText.style.color = "#aaa";
    chrome.storage.local.remove(["bloom.gifBackgroundUrl"]);
  }
}

function previewGifBackground() {
  const url = gifBackgroundUrlInput.value.trim();
  const opacity = backgroundOpacitySlider ? backgroundOpacitySlider.value : 0.5;

  if (url) {
    if (isValidUrl(url)) {
      previewGifImage.src = url;
      previewGifImage.style.display = "block";
      previewGifText.textContent = "GIF Preview";
      
      previewGifImage.style.filter = `brightness(${opacity})`;

      previewGifImage.onerror = () => {
        previewGifImage.style.display = "none";
        previewGifText.textContent = "Error: Unable to load GIF from URL";
        previewGifText.style.color = "#f93d3d";
      };

      previewGifImage.onload = () => {
        previewGifText.style.color = "white";
      };
      
      if (backgroundImageInput.value.trim()) {
        backgroundImageInput.value = "";
        previewImage.style.display = "none";
        previewText.textContent = "Static image disabled when using GIF background";
        previewText.style.color = "#aaa";
        chrome.storage.local.remove(["bloom.backgroundImage"]);
      }
    } else {
      previewGifImage.style.display = "none";
      previewGifText.textContent = "Invalid URL format";
      previewGifText.style.color = "#f93d3d";
    }
  } else {
    previewGifImage.style.display = "none";
    previewGifText.textContent = "No GIF background set";
    previewGifText.style.color = "white";
  }
}

function isValidUrl(string) {
  try {
    new URL(string);
    return true;
  } catch (_) {
    return false;
  }
}

function updateOpacitySlider() {
  const value = backgroundOpacitySlider.value;

  const formattedValue = Number(value).toFixed(2);
  opacityValueDisplay.textContent = formattedValue;

  if (previewImage.style.display === 'block') {
    previewImage.style.filter = `brightness(${value})`;
  }
  
  if (previewGifImage.style.display === 'block') {
    previewGifImage.style.filter = `brightness(${value})`;
  }
}